/*
 * gaia - opensource 3D interface to the planet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/*
Simple program to fetch single image from Google Earth server.
*/

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <gefetch.h>

int main(int argc, char **argv) {
	int f;
	gefetch_t gef;

	if ((gef = gefetch_init()) == 0) {
		fprintf(stderr, "Unable to init libgefetch\n");
		return 1;
	}

	if (gefetch_fetch_image(gef, 0, 0, 0) != GEFETCH_OK) {
		fprintf(stderr, "Unable to fetch image\n");
		gefetch_cleanup(gef);
		return 2;
	}

	if ((f = open("0.jpg", O_CREAT | O_TRUNC | O_WRONLY, 0644)) < 0) {
		fprintf(stderr, "Unable to open 0.jpg for writing\n");
		gefetch_cleanup(gef);
		return 3;
	}

	if (write(f, gefetch_get_data_ptr(gef), gefetch_get_data_size(gef)) != gefetch_get_data_size(gef)) {
		fprintf(stderr, "Unable write %d bytes\n", gefetch_get_data_size(gef));
		gefetch_cleanup(gef);
		close(f);
		return 4;
	}

	fprintf(stderr, "0.jpg fetched successfully\n");

	close(f);
	gefetch_cleanup(gef);

	return 0;
}
